<?php

/*
Plugin Name: Timeliner for WordPress
Plugin URI: http://www.nikolaydyankov.com/
Version: 2.0.6
Author: Nikolay Dyankov
Description: Generates an interactive timeline from selected posts.
*/

if (!class_exists('Timeliner')) {
	class Timeliner {
		function __construct() {
			$this->admin_options_name = 'timeline-admin-options';
			$this->defaults = array();
			$this->pagename = 'timeliner';
			$this->new_pagename = 'timeline-new';
			$this->default_settings = array(
				'width' => 400,
				'auto_width' => 1,
				'show_latest' => 1,
				'select_latest' => 1,
				'query-mode' => 'tax',
				'query' => array(
					'tax_name' => 'category',
					'tax_terms' => array(1),
					'posts_per_page' => 20
				)
			);
		}
		function get_admin_options() {
			// delete_option($this->admin_options_name);

			$loaded_options = get_option($this->admin_options_name);

			if (!empty($loaded_options)) {
				foreach ($loaded_options as $key => $option) {
					$admin_options[$key] = $option;
				}
			} else {
				$admin_options = array();
			}


			update_option($this->admin_options_name, $admin_options);
			return $admin_options;
		}
		function init_pages() {
			add_menu_page(
				"Timeliner",
				"Timeliner",
				"manage_options",
				$this->pagename,
				array($this, 'print_options_page')
			);

			add_submenu_page(
				$this->pagename,
				"Add New Timeline",
				"Add New",
				"manage_options",
				$this->new_pagename,
				array($this, 'print_instance_options')
			);
		}

		function admin_includes() {
			wp_enqueue_script('timeline-admin-js', plugins_url('/js/admin/admin.js', __FILE__), false, '1.0', true);
			wp_enqueue_style('timeline-admin-css', plugins_url('/css/admin/admin.css', __FILE__), false, '1.0', false);
			wp_enqueue_style('font-awesome-css', plugins_url('/css/lib/font-awesome.css', __FILE__), false, '1.0', false);
		}
		function user_includes() {
			wp_enqueue_script('jquery');
			wp_register_script('timeliner-js', plugins_url('/js/client/timeliner.js', __FILE__), false, '1.0', true);
			wp_enqueue_script('timeliner-js');
			wp_register_style('timeliner-css', plugins_url('/css/client/timeliner.css', __FILE__), false, '1.0', false);
			wp_enqueue_style('timeliner-css');
		}

		function print_options_page() {
			$options = $this->get_admin_options();

			if (isset($_GET['action'])) {
				if ($_GET['action'] == 'edit') {
					$this->print_instance_options();
				}
				if ($_GET['action'] == 'delete') {
					$this->delete_instance();
					$this->print_main_options();
				}
			} else {
				$this->print_main_options();
			}
		}
		function print_main_options() {
			$options = $this->get_admin_options();

			?>

			<div class="as">
				<header>
					<img src="<?php echo plugins_url('images/thumb.png', __FILE__); ?>">
					<h2>WordPress</h2>
					<h1>Timeliner</h1>
				</header>

				<div class="as-c">

					<?php if (!isset($options['timelines']) || count($options['timelines']) == 0) { ?>
						<div class="greetings">Hey! Seems like you haven't created any timelines yet. Start by clicking the big green button below!</div>
					<?php } else { ?>
					<table class="ndd-clean-table ndd-yellow-rows">
						<thead>
							<tr>
								<th class="ndd-column-name">Timeline Name</th>
								<th class="ndd-column-shortcode">Shortcode</th>
								<th class="ndd-column-delete">Delete</th>
							</tr>
						</thead>
						<tbody>

							<?php
								if (isset($options['timelines']) && $options['timelines']) {
									foreach ($options['timelines'] as $timeline) {
										$yellow_class = '';
										if ($timeline['new']) {
											$yellow_class = 'ndd-new-row';
										}
										$options['timelines'][$timeline['id']]['new'] = false;
										update_option($this->admin_options_name, $options);
							?>

							<tr class="<?php echo $yellow_class; ?>">
								<td class="ndd-column-name"><a href="?page=<?php echo $this->pagename; ?>&id=<?php echo $timeline['id']; ?>&action=edit"><?php echo $timeline['title']; ?></a></td>
								<td class="ndd-column-shortcode"><a href="?page=<?php echo $this->pagename; ?>&id=<?php echo $timeline['id']; ?>&action=edit">[timeline name="<?php echo $timeline['shortcode']; ?>"]</a></td>
								<td class="ndd-column-delete"><div class="ndd-delete-cell-wrap"><a href="?page=<?php echo $this->pagename; ?>&id=<?php echo $timeline['id']; ?>&action=delete" class="ndd-delete-row"></a></div></td>
							</tr>

							<!-- END FOREACH -->
							<?php 	} ?>
							<?php } ?>

						</tbody>
					</table>

					<!-- END ELSE -->
					<?php }  ?>

					<a href="?page=<?php echo $this->new_pagename; ?>" class="ndd-button-green-regular"><span class="icon-plus"></span> New Timeline</a>
				</div>

				<footer>
					<p class="footer-copy">Thank you for purchasing my product. If you like what I do and you'd like to support my work, the best way to do it is by rating my items on CodeCanyon. You can do that by going at <a href="http://codecanyon.net/">CodeCanyon</a> and clicking on the Downloads tab. Find my product there and click on the stars to give it a rating. Thank you!</p>
					<ul>
						<li><span>Designed and developed by</span><a href="http://www.nikolaydyankovdesign.com"><img src="<?php echo plugins_url('images/admin/my-logo.png', __FILE__); ?>"></a></li>
						<li><span>Available exclusively at</span><a href="http://www.codecanyon.com/?ref=nickys"><img src="<?php echo plugins_url('images/admin/codecanyon-logo.png', __FILE__); ?>"></a></li>
						<li><span>Customer Support</span><a href="http://support.nikolaydyankovdesign.com"><img src="<?php echo plugins_url('images/admin/support.png', __FILE__); ?>"></a></li>
					</ul>
				</footer>
			</div>

			<?php

			$this->admin_includes();
		}
		function print_instance_options() {
			if (isset($_POST['save_options'])) {
				$this->save_options();
			}

			$options = $this->get_admin_options();

			// Change the title of the page if action is "New" or "Edit"

			if ($_GET['page'] == $this->new_pagename) {
				$submit_name = 'Create Timeline';
				$pagetitle = 'New Timeline';
				$title = '';
				$shortcode = '';
				$id = rand(0, 10000);

				$settings = $this->default_settings;
			} else {
				$submit_name = 'Save Options';
				$id = $_GET['id'];
				$timeline = $options['timelines'][$id];
				$pagetitle = 'Edit Timeline';
				$title = $timeline['title'];
				$shortcode = $timeline['shortcode'];

				$settings = $timeline['settings'];
			}

			$uri = $_SERVER["REQUEST_URI"];
			$uri_ar = explode('?', $uri);

			$uri = $uri_ar[0] . '?page=' . $this->pagename . '&id=' . $id . '&action=edit';

			?>
			<form action="<?php echo $uri; ?>" method="post">
			<div class="as">
				<header class="ndd-subpage-header">
					<a href="?page=<?php echo $this->pagename; ?>" class="ndd-back-link">Back</a>
					<h1><?php echo $pagetitle; ?></h1>
					<div class="ndd-button-submit-wrap">

						<!-- OUTPUT FIELDS -->

						<input type="text" class="ndd-invisible" id="query-mode" name="query-mode">
						<input type="text" class="ndd-invisible" id="query-post-ids" name="query-post-ids">

						<input type="text" class="ndd-invisible" id="query-tax-name" name="query-tax-name">
						<input type="text" class="ndd-invisible" id="query-tax-terms" name="query-tax-terms">

						<input type="text" class="ndd-invisible" id="query-number-posts" name="query-number-posts">

						<!--  -->

						<input class="ndd-button-submit" name="save_options" type="submit" class="button-primary" id="save_options" value="<?php echo $submit_name; ?>">
					</div>
					<?php if (isset($_POST['save_options'])) { ?>
						<div class="updated"><p><strong> <?php echo _e("Settings Updated!", "CommentWordFilter"); ?> </strong></p></div>
					<?php } ?>
				</header>

				<div class="as-c">
					<div class="ndd-tab-group">
						<ul>
							<li class="active">General</li>
							<li>Content</li>
							<li>Settings</li>
						</ul>

						<div class="ndd-tab active">
							<?php
								if ($pagetitle == 'New Timeline') {
									echo '<div class="greetings">Start by filling in the title and shortcode, then head over to the Content and Settings tabs above to further setup your timeline.</div>';
								}
							?>
							<table class="form-table ndd-yellow-rows">
								<tr>
									<td>
									<h2><label for="title">Title</label></h2>
									<input type="text" name="title" id="title" value="<?php echo $title; ?>" class="ndd-large-input">
									<div class="ndd-form-help">The title is used only for convenience in the admin panel. It will not be visible for the page visitor.</div>
									</td>
								</tr>
								<tr>
									<td>
									<h2><label for="shortcode">Shortcode</label></h2>
									<input type="text" name="shortcode" id="shortcode" value="<?php echo $shortcode; ?>" class="ndd-large-input">
									<div class="ndd-form-help">This is the shortcode that you will use to include this timeline in a page or a template.</div>
									</td>
								</tr>
							</table>
						</div>
						<div class="ndd-tab">
							<div id="select-feed-type" class="ndd-content-step ndd-content-step-1 active">
								<h2>1. Which posts to show?</h2>
								<div class="ndd-radio-group-2-cols ndd-radio-group">
									<ul>
										<li id="most-recent-posts" <?php if ($settings['query-mode'] == 'tax') echo 'class="active"'; ?>><span>Most Recent Posts</span></li>
										<li id="selected-posts" <?php if ($settings['query-mode'] == 'posts') echo 'class="active"'; ?>><span>Selected Posts</span></li>
									</ul>
								</div>
								<div class="ndd-form-help">"Most Recent Posts" will automatically pull posts from the database based on a specific criteria -
								category, tag, or other.</div>
							</div>

							<!-- Select each post individually -->
							<div id="select-posts" class="ndd-content-step ndd-content-step-2  <?php if ($settings['query-mode'] == 'posts') echo 'active'; ?>">
								<h2>2. Which Posts?</h2>
								<div class="ndd-radio-group-cloud ndd-radio-group ndd-select-multiple">
									<ul>
										<?php
											query_posts(array(
												"posts_per_page" => -1
											));
											if (have_posts()) {
												while (have_posts()) {
													the_post();
													$class = '';

													if ($settings['query-mode'] == 'posts') {
														if (array_search(get_the_id(), $timeline['settings']['query']['post__in']) !== false) {
															$class = ' class="active"';
															$post_is_selected = true;
														}
													}


													echo '<li data-id="' . get_the_id() . '"' . $class . '>' . get_the_title() . '</li>';
												}
											}
										?>
									</ul>
									<div class="clear"></div>
								</div>
								<div class="ndd-form-help">Hold SHIFT to select multiple posts.</div>
							</div>

							<div id="select-posts-number-posts" class="ndd-content-step ndd-content-step-3 <?php  if ($settings['query-mode'] == 'posts' && $post_is_selected) echo 'active'; ?>">
								<h2>3. How many posts?</h2>
								<div class="ndd-radio-group-4-cols ndd-radio-group">
									<ul>
										<li <?php if ($settings['query']['posts_per_page'] == 5) echo 'class="active"'; ?>>5</li>
										<li <?php if ($settings['query']['posts_per_page'] == 10) echo 'class="active"'; ?>>10</li>
										<li <?php if ($settings['query']['posts_per_page'] == 20) echo 'class="active"'; ?>>20</li>
										<li <?php if ($settings['query']['posts_per_page'] == 50) echo 'class="active"'; ?>>50</li>
									</ul>
								</div>
								<div class="ndd-form-help">This is the total number of posts that will be in the timeline.</div>
							</div>

							<!-- Select posts by category, tag or custom taxonomy -->
							<div id="select-feed-source" class="ndd-content-step ndd-content-step-2 		<?php if ($settings['query-mode'] == 'tax') echo 'active'; ?>">
								<h2>2. Select by Category, tag, or custom taxonomy?</h2>
								<div class="ndd-radio-group-3-cols ndd-radio-group">
									<ul>
										<li id="select-category-button" <?php if ($settings['query-mode'] == 'tax' && $settings['query']['tax_name'] == 'category') echo 'class="active"'; ?>>Category</li>
										<li id="select-tag-button" <?php if ($settings['query-mode'] == 'tax' && $settings['query']['tax_name'] == 'post_tag') echo 'class="active"'; ?>>Tag</li>
										<li id="select-custom-tax-button" <?php if ($settings['query-mode'] == 'tax' && $settings['query']['tax_name'] != 'post_tag' && $settings['query']['tax_name'] != 'category') echo 'class="active"'; ?>>Custom Taxonomy</li>
									</ul>
								</div>
								<div class="ndd-form-help">Select "Other" if you want to get posts by a custom taxonomy.</div>
							</div>

							<div id="select-category" class="ndd-content-step ndd-content-step-3  			<?php if ($settings['query-mode'] == 'tax' && $settings['query']['tax_name'] == 'category'	) echo 'active'; ?>">
								<h2>3. Which Categories?</h2>
								<div class="ndd-radio-group-cloud ndd-radio-group ndd-select-multiple">
									<ul>
										<?php
											$cats = get_categories();

											foreach ($cats as $cat) {
												$class = '';

												if ($settings['query-mode'] == 'tax') {

													if (array_search($cat->cat_ID, $settings['query']['tax_terms']) !== false) {
														$class = ' class="active"';
														$cat_is_selected = true;
													}
												}

												echo '<li'. $class .' data-id="' . $cat->cat_ID . '">' . $cat->name . '</li>';
											}
										?>
									</ul>
									<div class="clear"></div>
								</div>
								<div class="ndd-form-help">Hold SHIFT to select multiple categories.</div>
							</div>

							<div id="select-tag" class="ndd-content-step ndd-content-step-3  				<?php if ($settings['query-mode'] == 'tax' && $settings['query']['tax_name'] == 'post_tag'	) echo 'active'; ?>">
								<h2>3. Which Tags?</h2>
								<div class="ndd-radio-group-cloud ndd-radio-group ndd-select-multiple">
									<ul>
										<?php
											$tags = get_tags();

											foreach ($tags as $tag) {

												if ($settings['query-mode'] == 'tax') {
													$class = '';
													if (array_search($tag->name, $timeline['settings']['query']['tax_terms']) !== false) {
														$class = ' class="active"';
														$tag_is_selected = true;
													}
												}

												echo '<li'. $class .' data-name="' . $tag->name . '">' . $tag->name . '</li>';
											}
										?>

										<?php
											// $tags = get_tags();
											// foreach ($tags as $tag) {
											// 	echo '<li>' . $tag->name . '</li>';
											// }
										?>
									</ul>
									<div class="clear"></div>
								</div>
								<div class="ndd-form-help">Hold SHIFT to select multiple tags.</div>
							</div>

							<div id="select-custom-tax" class="ndd-content-step ndd-content-step-3  		<?php if ($settings['query-mode'] == 'tax' && $settings['query']['tax_name'] != 'post_tag' 	&& $settings['query']['tax_name'] != 'category') echo 'active'; ?>">
								<h2>3. Select Custom Taxonomy</h2>
								<div class="ndd-radio-group-cloud ndd-radio-group">
									<ul>
										<?php
											$taxonomies = get_taxonomies('', 'names');
											foreach ($taxonomies as $taxonomy) {
												if ($taxonomy == 'category') continue;
												if ($taxonomy == 'post_tag') continue;
												if ($settings['query-mode'] == 'tax') {
													$class = '';
													if ($taxonomy == $settings['query']['tax_name']) {
														$class = ' class="active"';
														$tax_is_selected = true;
													}
												}

												echo '<li id="taxonomy-button-' . $taxonomy . '"' . $class . '>'. $taxonomy. '</li>';
											}
										?>
									</ul>
									<div class="clear"></div>
								</div>
								<div class="ndd-form-help">In this step you select custom taxonomies. <br />What is a taxonomy? A taxonomy is a way to group posts together. You can group them by category or tags for example. So "category" and "tag" are taxonomies. And "custom" taxonomies means that they have been added by the theme developer.</div>
							</div>

							<div id="select-custom-terms" class="ndd-content-step ndd-content-step-4 		<?php if ($settings['query-mode'] == 'tax' && $tax_is_selected 							&& $settings['query']['tax_name'] != 'category' 												&& $settings['query']['tax_name'] != 'post_tag') echo 'active'; ?>">
								<h2>4. Select Taxonomy Term</h2>
								<div class="ndd-radio-group-cloud ndd-radio-group ndd-select-multiple">

									<?php
										$taxonomies = get_taxonomies('', 'names');
										foreach ($taxonomies as $taxonomy) {
											$active = '';

											if (isset($settings['query']['tax_name']) && $taxonomy == $settings['query']['tax_name']) {
												$active = ' active';
											}

											echo '<ul class="taxonomy-terms-list' . $active . '" id="taxonomy-terms-list-' . $taxonomy . '">';
											$terms = get_terms($taxonomy);

											foreach ($terms as $term) {
												if ($settings['query-mode'] == 'tax') {
													$class = '';

													if (array_search($term->slug, $timeline['settings']['query']['tax_terms']) !== false) {
														$class = ' class="active"';
														$term_is_selected = true;
													}
												}

												echo '<li data-id="' . $term->slug . '"' . $class . '>'. $term->name. '</li>';
											}

											echo '</ul>';
										}
									?>

									<div class="clear"></div>
								</div>
								<div class="ndd-form-help">Hold SHIFT to select multiple terms.</div>
							</div>

							<div id="select-custom-number-posts" class="ndd-content-step ndd-content-step-5 <?php if ($settings['query-mode'] == 'tax' && $term_is_selected 						&& $settings['query']['tax_name'] != 'category' 												&& $settings['query']['tax_name'] != 'post_tag') echo 'active'; ?>">
								<h2>5. How many posts?</h2>
								<div class="ndd-radio-group-4-cols ndd-radio-group">
									<ul>
										<li <?php if ($settings['query']['posts_per_page'] == 5) echo 'class="active"'; ?>>5</li>
										<li <?php if ($settings['query']['posts_per_page'] == 10) echo 'class="active"'; ?>>10</li>
										<li <?php if ($settings['query']['posts_per_page'] == 20) echo 'class="active"'; ?>>20</li>
										<li <?php if ($settings['query']['posts_per_page'] == 50) echo 'class="active"'; ?>>50</li>
									</ul>
								</div>
								<div class="ndd-form-help">This is the total number of posts that will be in the timeline.</div>
							</div>

							<div id="select-number-posts" class="ndd-content-step ndd-content-step-4 		<?php if ($settings['query-mode'] == 'tax' && ($tag_is_selected || $cat_is_selected) 	&& ($settings['query']['tax_name'] == 'category' || $settings['query']['tax_name'] == 'post_tag')	) echo 'active'; ?>">
								<h2>4. How many posts?</h2>
								<div class="ndd-radio-group-4-cols ndd-radio-group">
									<ul>
										<li <?php if ($settings['query']['posts_per_page'] == 5) echo 'class="active"'; ?>>5</li>
										<li <?php if ($settings['query']['posts_per_page'] == 10) echo 'class="active"'; ?>>10</li>
										<li <?php if ($settings['query']['posts_per_page'] == 20) echo 'class="active"'; ?>>20</li>
										<li <?php if ($settings['query']['posts_per_page'] == 50) echo 'class="active"'; ?>>50</li>
									</ul>
								</div>
								<div class="ndd-form-help">This is the total number of posts that will be in the timeline.</div>
							</div>
						</div>
						<div class="ndd-tab">
							<!--
								SETTINGS
							-->

							<table class="form-table ndd-yellow-rows">
								<tr valign="top">
									<th scope="row"><label for="width">Width</label></th>
									<td>
										<input type="text" class="regular-text" name="width" id="width" value="<?php echo $settings['width']; ?>" <?php if ($settings['auto_width'] == true) echo 'disabled'; ?>>
										<input type="checkbox" class="regular-text" name="auto_width" id="auto_width" <?php if ($settings['auto_width'] == true) echo 'checked'; ?> style="display: inline-block; width: auto;">
										<label for="auto_width">Auto</label>
										<div class="ndd-form-help">The total width of the timeline in pixels. Checking "Auto" will make the timeline as wide as it's parent element, but it will still be static, not fluid.</div>
									</td>
								</tr>
								<tr valign="top">
									<th scope="row"><label for="show_latest">Show Latest Post</label></th>
									<td><input type="checkbox" class="regular-text" name="show_latest" id="show_latest" <?php if ($settings['show_latest']) echo 'checked'; ?>><div class="ndd-form-help">Check this if you want the latest post to be shown on page load.</div></td>
								</tr>
								<tr valign="top">
									<th scope="row"><label for="select_latest">Select Latest Post</label></th>
									<td><input type="checkbox" class="regular-text" name="select_latest" id="select_latest" <?php if ($settings['select_latest']) echo 'checked'; ?>><div class="ndd-form-help">Check this if you want the latest post to be selected on page load. Doesn't have an effect unless "Show Latest Post" is checked.</div></td>
								</tr>
							</table>
						</div>
					</div>

				</div>

				<footer>
					<p class="footer-copy">Hey! Thank you for purchasing my product. If you like what I do and you'd like to support my work, the best way to do it is by rating my items on CodeCanyon. You can do that by going at <a href="http://codecanyon.net/">CodeCanyon</a> and clicking on the Downloads tab. Find my product there and click on the stars to give it a rating. Thank you!</p>
					<ul>
						<li><span>Designed and developed by</span><a href="http://www.nikolaydyankovdesign.com"><img src="<?php echo plugins_url('images/admin/my-logo.png', __FILE__); ?>"></a></li>
						<li><span>Available exclusively at</span><a href="http://www.codecanyon.com/?ref=nickys"><img src="<?php echo plugins_url('images/admin/codecanyon-logo.png', __FILE__); ?>"></a></li>
						<li><span>Customer Support</span><a href="http://support.nikolaydyankovdesign.com"><img src="<?php echo plugins_url('images/admin/support.png', __FILE__); ?>"></a></li>
					</ul>
				</footer>

				</form>
			</div>

			<?php

			$this->admin_includes();
		}

		function save_options() {
			$options = $this->get_admin_options();
			$id = $_GET['id'];

			if (!isset($_GET['id'])) {
				$id = str_replace(' ', '-', strtolower($_POST['title']));
				$timeline = array();
			} else {
				if (!isset($options['timelines'][$id])) {
					$timeline = array();
				} else {
					$timeline = $options['timelines'][$id];
				}
			}

			if (!isset($timeline['id'])) {
				$timeline['new'] = true;
				$timeline['settings'] = $this->default_settings;
			}


			$timeline['id'] = $id;
			$timeline['title'] = $_POST['title'];
			$timeline['shortcode'] = $_POST['shortcode'];


			// ============ Settings

			if (isset($_POST['width'])) {
				$timeline['settings']['width'] = $_POST['width'];
			}
			if (isset($_POST['auto_width'])) {
				$timeline['settings']['auto_width'] = 1;
			} else {
				$timeline['settings']['auto_width'] = 0;
			}
			if (isset($_POST['show_latest'])) {
				$timeline['settings']['show_latest'] = 1;
			} else {
				$timeline['settings']['show_latest'] = 0;
			}
			if (isset($_POST['select_latest'])) {
				$timeline['settings']['select_latest'] = 1;
			} else {
				$timeline['settings']['select_latest'] = 0;
			}

			//  =========== Query Options

			if (isset($_POST['query-mode'])) {
				$timeline['settings']['query-mode'] = $_POST['query-mode'];
			}
			$timeline['settings']['query'] = array();

			// Number of posts
			if (isset($_POST['query-number-posts'])) {
				$timeline['settings']['query']['posts_per_page'] = $_POST['query-number-posts'];
			}

			// Posts mode
			if ($timeline['settings']['query-mode'] == 'posts') {
				$timeline['settings']['query']['post__in'] = explode(' ', $_POST['query-post-ids']);

				// trim the last element
				if ($timeline['settings']['query']['post__in'][count($timeline['settings']['query']['post__in']) - 1] == '') {
					unset($timeline['settings']['query']['post__in'][count($timeline['settings']['query']['post__in']) - 1]);
				}
			}

			// Taxonomy mode

			if ($timeline['settings']['query-mode'] == 'tax') {
				$timeline['settings']['query']['tax_name'] = $_POST['query-tax-name'];

				$timeline['settings']['query']['tax_terms'] = explode('%', $_POST['query-tax-terms']);

				// trim the last element
				if (end($timeline['settings']['query']['tax_terms']) == NULL) {
					unset($timeline['settings']['query']['tax_terms'][count($timeline['settings']['query']['tax_terms']) - 1]);
				}
			}

			$this->do_query($timeline['settings']['query'], $timeline['settings']['query-mode']);
			$timeline['content'] = $this->get_html_content($timeline['id'], $timeline['settings']['auto_width'], $timeline['settings']['width']);

			$options['timelines'][$id] = $timeline;
			update_option($this->admin_options_name, $options);
		}
		function delete_instance() {
			$options = $this->get_admin_options();
			unset($options['timelines'][$_GET['id']]);
			update_option($this->admin_options_name, $options);
		}
		function do_query($args, $mode) {
			if (!isset($args['tax_name'])) {
				$args['tax_name'] = '';
			}

			if (!isset($args['tax_terms'])) {
				$args['tax_terms'] = '';
			}

			// Do query
			if ($mode == 'posts') {
				// Posts mode
				query_posts(array(
					'posts_per_page' => $args['posts_per_page'],
					'post__in' => $args['post__in']
				));
			} else {
				if ($args['tax_name'] == 'category') {

					// Category mode
					$cats = '';

					for ($i=0; $i<count($args['tax_terms']); $i++) {
						$cats .= $args['tax_terms'][$i] . ',';
					}

					$cats = substr($cats, 0, strlen($cats) - 1);
					query_posts('posts_per_page=' . $args['posts_per_page'] . '&cat=' . $cats);

				} else if ($args['tax_name'] == 'post_tag') {

					// Tag mode
					$tags = '';

					for ($i=0; $i<count($args['tax_terms']); $i++) {
						$tags .= $args['tax_terms'][$i] . ',';
					}

					$tags = substr($tags, 0, strlen($tags) - 1);
					query_posts('posts_per_page=' . $args['posts_per_page'] . '&tag=' . $tags);
					// query_posts('tag=black,red');
				} else {
					// Custom taxonomy mode
					$terms = '';

					for ($i=0; $i<count($args['tax_terms']); $i++) {
						if (isset($args['tax_terms'][$i])) {
							$terms .= $args['tax_terms'][$i] . ',';
						}
					}

					$terms = substr($terms, 0, strlen($terms) - 1);

					query_posts(array(
						'tax_query' => array(
							array(
								'taxonomy' => $args['tax_name'],
								'field'    => 'slug',
								'terms'    => $args['tax_terms']
							)
						),
						'posts_per_page' => $args['posts_per_page']
					));
				}
			}
		}
		function get_featured_image() {
			global $post, $posts;

			if (has_post_thumbnail()) {
				return wp_get_attachment_url(get_post_thumbnail_id($post->ID));
			}

			$first_img = '';
			ob_start();
			ob_end_clean();
			$output = preg_match_all('/<img.+src=[\'"]([^\'"]+)[\'"].*>/i', $post->post_content, $matches);

			if (isset($matches[1][0])) {
				$first_img = $matches[1][0];
			}

			if(empty($first_img)){ //Defines a default image
				$first_img = '';
			}

			return $first_img;
		}
		function update_data() {
			$options = $this->get_admin_options();

			if ($options['timelines']) {
				foreach ($options['timelines'] as $timeline) {
					$id = $timeline['id'];

					$this->do_query($timeline['settings']['query'], $timeline['settings']['query-mode']);
					$timeline['content'] = $this->get_html_content($timeline['id'], $timeline['settings']['auto_width'], $timeline['settings']['width']);

					$options['timelines'][$id] = $timeline;
				}
			}

			update_option($this->admin_options_name, $options);
		}
		function get_html_content($id, $auto, $width) {
			$content = '';

			if ($auto != 1) {
				$content .= '<div class="timeline-outer-wrap" style="width: ' . $width . 'px; position: relative; z-index: 9999;">';
			}

			$content .= '<div class="timeliner-wp" id="timeline-' . $id . '" style="position: relative; z-index: 9999;"></div>';
			$content .= '	<div id="timeline-' . $id . '-src" style="display: none;">';

			while(have_posts()) {
				the_post();
				$content .= '	<div class="timeline-event">';
				$content .= '	<div class="timeline-date">' . get_the_date('d.m.Y') . '</div>';
				$content .= '	<div class="timeline-title">' . get_the_title() . '</div>';
				$content .= '	<div class="timeline-thumb">' . $this->get_featured_image() . '</div>';
				$content .= '	<div class="timeline-content">' . strip_tags(substr(get_the_content(), 0, 1000)) . '</div>';
				$content .= '	<div class="timeline-link"><a href="' . get_permalink() . '">Read More</a></div>';
				$content .= '</div>';
			}

			$content .= '</div>';

			if ($auto != 1) {
				$content .= '</div>';
			}

			return $content;
		}

		function shortcodes() {
			$options = $this->get_admin_options();

			if (isset($options['timelines']) && count($options['timelines'])) {
				foreach ($options['timelines'] as $timeline) {
					add_shortcode('timeline', array($this, 'print_shortcode'));
				}
			}
		}
		function print_shortcode($atts) {
			$options = $this->get_admin_options();
			$shortcode = $atts['name'];

			if ($options['timelines']) {
				foreach($options['timelines'] as $timeline) {
					if ($timeline['shortcode'] == $shortcode) {
						$result = $timeline;
					}
				}
				return $result['content'];
			}
		}

		function call_plugins() {
			$options = $this->get_admin_options();

			?>

			<script>
				(function($, undefined) {
					$(document).ready(function() {

						<?php
							if ($options['timelines']) {
								foreach($options['timelines'] as $timeline) {
									$src = '$("#timeline-' . $timeline['id'] . '-src")';
									$show_latest = $timeline['settings']['show_latest'];
									$select_latest = $timeline['settings']['select_latest'];
						?>

						$('#timeline-<?php echo $timeline['id']; ?>').timelinexml({
							'src' : <?php echo $src; ?>,
							'show_latest' : <?php echo $show_latest; ?>,
							'select_latest' : <?php echo $select_latest; ?>,
						});

						<?php 	} ?>
						<?php } ?>
					});
				})(jQuery);
			</script>

			<?php
		}
	}
}

if (class_exists('Timeliner')) {
	$timeline = new Timeliner();
	$timeline->shortcodes();
}

add_action('wp_footer', array($timeline, 'call_plugins'));
add_action('admin_menu', array($timeline, 'init_pages'));
add_action('wp_enqueue_scripts', array($timeline, 'user_includes'));

add_filter('publish_post', array($timeline, 'update_data'));
add_filter('trash_post', array($timeline, 'update_data'));
add_filter('edit_post', array($timeline, 'update_data'));
add_filter('edit_category', array($timeline, 'update_data'));
add_filter('publish_post', array($timeline, 'update_data'));
add_filter('private_to_publish', array($timeline, 'update_data'));
add_filter('save_post', array($timeline, 'update_data'));
?>
